require 'net/http'
require 'nokogiri'

# ダウンロードファイルを保存するディレクトリを作成する
download_dir = 'images'
Dir.mkdir(download_dir) unless Dir.exist?(download_dir)
# ダウンロードディレクトリに移動する
Dir.chdir(download_dir) do
  # Wikimedia Commonsで「Ruby」というキーワードで検索した結果のURL
  url = URI.parse('https://commons.wikimedia.org/w/index.php?
search=Ruby&title=Special:MediaSearch&go=Go&type=image')
  doc = Nokogiri::HTML5(Net::HTTP.get(url))

  # class属性がsdms-search-resultsである要素の子孫である、
    # src属性を持ったimgタグを抽出する
  doc.css('.sdms-search-results img[src]').each do |element|
    begin
      # URLにスペースが含まれているとURI.parseでエラーになるため、+に置換
      image_url = URI.parse(element.attr('src').gsub(' ', '+'))
      # URLが相対パスの場合、元のWebサイトのURLに連結する
      unless image_url.absolute?
        image_url = url + image_url
      end

      puts "ダウンロード中: #{image_url}"
      image = Net::HTTP.get(image_url)
    rescue => e
      # URI.parseやNet::HTTP.getで例外が発生したらスキップして次に進む
      puts "ダウンロードに失敗しました: #{e.message}"
      next
    end
    # URLの一番最後にある/以降の部分をファイル名とする
    filename = File.basename(image_url.path)
    # ファイルをバイナリモードで開いて保存する
    File.binwrite(filename, image)
    puts "保存完了: #{filename}"
  end
end
